#!/bin/sh /usr/share/centrifydc/perl/run

##############################################################################
#
# Copyright (C) 2014-2020 Centrify Corporation. All rights reserved.
#
##############################################################################


use strict;
use lib '/usr/share/centrifydc/perl';
use CentrifyDC::Include;
use CentrifyDC::GP::Mapper;
use CentrifyDC::GP::General qw(:debug IsFipsSupported);
use CentrifyDC::Platform qw(GetPlatform OS_FAMILY OS ARCH);

my $file;
my $action;
my $user;
$file = {
    'comment_markers' => [
      '#',
    ],
    'hierarchy_separator' => '.',
    'list_expr' => ', *| +',
    'list_separator' => ', ',
    'lock' => '/etc/centrifydc/centrifydc.conf.lock',
    'match_expr' => [
      '/^\s*([^\s:=]+)[:=]\s*(.*)/',
    ],
    'named_list_separator' => ',',
    'parent_expr' => '^(.*)\.([^\.]+)$',
    'path' => [
      '/etc/centrifydc/centrifydc.conf',
    ],

    'value_map' => {

      'fips.mode.enable' => {
        'default_data' => 'false',
        'post_action' => [
          'DO_RESTART_ADCLIENT_AND_EXPIRE_CACHE',
        ],
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDC/Settings/Fips',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'fips.mode.enable',
        'value_type' => 'named',
        'valueoff' => 'false',
        'valueon' => 'true',
      },

    },

    'write_data' => '$value: $data\n',
};

$action = $ARGV[0];
my $mode = $ARGV[2] ? $ARGV[2] : $ARGV[1];
$user = $ARGV[2] ? $ARGV[1] : undef;

IsFipsSupported() or exit(0);

if ($action eq "unmap")
{

    CentrifyDC::GP::Mapper::UnMap($file, $user);
}
else
{
    CentrifyDC::GP::Mapper::Map($file, $user);
}
