#!/bin/bash

# Set Paths
CIMS_USR=/usr
CIMS_INSTALL=/usr/share/centrifydc

# usage
usage () {
    echo "usage: `basename $0` enable|disable [module_name]" > /dev/stderr
}

# check redhat
is_redhat() {
    return [ -f /etc/redhat-release ]
}

# get status
do_sctool_status() {
    sctool -s > /dev/null 2>&1
    return $?
}

# check status
check_sctool_status() {
    local OP=$1
    local RV_ENABLED=0
    local RV_DISABLED=2
    local ANS=

    if [ $OP == "enable" ]; then
        ANS=$RV_DISABLED
    else
        ANS=$RV_ENABLED
    fi

    do_sctool_status

    if [ $? -ne $ANS ]; then
        echo "Smart card support already ${OPERATION}d!" > /dev/stderr
        exit -2
    fi
}

# check root
check_root() {
    if [ `whoami` != "root" ]; then
        echo "Operation requires sudo privilege" > /dev/stderr
        exit -4
    fi
}

# check number of args
if [ $# -lt 1 ]; then
    usage
    exit 0
fi

ETC_PAM_PKCS11_DIR="/etc/pam_pkcs11"
PAM_PKCS11_FILE="$ETC_PAM_PKCS11_DIR/pam_pkcs11.conf"
PAM_PKCS11_BACKUP_FILE="$PAM_PKCS11_FILE.pre_cdc"

if [ `uname -i` == "i386" ]; then
    LIB="lib"
else
    LIB="lib64"
fi

SYSTEM_COOLKEY_SYMLINK="/usr/$LIB/libcoolkeypk11.so"
SYSTEM_COOLKEY_SYMLINK_BACKUP="$SYSTEM_COOLKEY_SYMLINK.pre_cdc"
SYSTEM_CKAPPLET_SYMLINK="/usr/$LIB/libckyapplet.so.1"
SYSTEM_CKAPPLET_SYMLINK_BACKUP="$SYSTEM_CKAPPLET_SYMLINK.pre_cdc"
CENTRIFY_COOLKEY_PATH="$CIMS_INSTALL/$LIB/pkcs11/libcoolkeypk11.so"
CENTRIFY_CKAPPLET_PATH="$CIMS_INSTALL/$LIB/libckyapplet.so.1.0.0"

NSSDB=/etc/pki/nssdb

COOLKEY_MODULE_NAME="CoolKey PKCS #11 Module"
CENTRIFY_PKCS11_MODULE="Centrify PKCS #11 Module"

# parse arg
OPERATION=$1
CENTRIFY_PKCS11_MODULE_PATH=${2:-$LIBPKCS11COOLKEY_FILE}
# If you change this, remember to change the one in lib/redhat/stuff.h as well
CENTRIFY_PKCS11_MODULE_SYMLINK="/usr/$LIB/pkcs11/libcentrifypkcs11.so"

# RHEL/CentOS major version, and smart card supported on 5 and above
RHEL_MAJOR_VERSION=7
RPM="/bin/rpm"
SED="/bin/sed"
RHEL_RPM_PACKAGE=`$RPM -q --whatprovides /etc/redhat-release`
if [ -n "$RHEL_RPM_PACKAGE" ]; then
    RHEL_MAJOR_VERSION=`$RPM -q --queryformat '%{VERSION}' $RHEL_RPM_PACKAGE | $SED -e 's#^\([0-9]*\).*#\1#'`
fi
# if get os major version failed, align to default 7.
if [ $RHEL_MAJOR_VERSION -lt 5 ]; then
    RHEL_MAJOR_VERSION=7
fi

# is this a valid operation?
case $OPERATION in
    enable)
        check_root

        # backup
        if [ ! -f $PAM_PKCS11_BACKUP_FILE ]; then
            mv $PAM_PKCS11_FILE $PAM_PKCS11_BACKUP_FILE > /dev/null
        fi

        # create temp dir
        TEMP_DIR=`perl \
            -e "use lib '$CIMS_INSTALL/perl';" \
            -e "use CentrifyDC::GP::General qw(GetTempDirPath);" \
            -e "print GetTempDirPath(0);" \
            | tail -n 1`

        if [ ! -d "$TEMP_DIR" ]; then
            echo "Failed to obtain temp dir path" > /dev/stderr
            exit -2
        fi

        TMPDIR=`mktemp -d $TEMP_DIR/tmp.XXXXX`

        # copy pam_pkcs11.conf and card_eventmgr.conf to /etc/pam_pkcs11
        cp -f $CIMS_INSTALL$ETC_PAM_PKCS11_DIR/pam_pkcs11.conf $ETC_PAM_PKCS11_DIR/.
        cp -f $CIMS_INSTALL$ETC_PAM_PKCS11_DIR/card_eventmgr.conf $ETC_PAM_PKCS11_DIR/.

        if [ $RHEL_MAJOR_VERSION -ge 8 ]; then
            sed -i 's/use_pkcs11_module = centrify;/use_pkcs11_module = opensc;/' $PAM_PKCS11_FILE
        fi

        # remove temp dir
        rm -rf $TMPDIR > /dev/null

        # handle smartcard-auth symlinks
        if [ -h /etc/pam.d/smartcard-auth ]; then
            if [ -f /etc/pam.d/smartcard-auth-ac ]; then
                rm /etc/pam.d/smartcard-auth
                cp --preserve=all /etc/pam.d/smartcard-auth-ac /etc/pam.d/smartcard-auth-ac.cdc
                if [ $RHEL_MAJOR_VERSION -lt 7 ]; then
                    /sbin/fixfiles restore /etc/pam.d/smartcard-auth-ac.cdc
                else
                    /sbin/restorecon /etc/pam.d/smartcard-auth-ac.cdc > /dev/null
                fi
                ln -s smartcard-auth-ac.cdc /etc/pam.d/smartcard-auth
            fi
        fi

        # unregister original CoolKey module, if any
        #COOLKEY_MODULE_NAME=`modutil -rawlist -dbdir /etc/pki/nssdb/ | grep libcoolkeypk11.so | perl -ne 'print if s/.*name=\"(.*)\"/\1/'`
        isThere=`modutil -rawlist -dbdir "$NSSDB" | grep "$COOLKEY_MODULE_NAME" || echo NO`
        if [ "$isThere" != "NO" ]; then
            modutil -delete "$COOLKEY_MODULE_NAME" -dbdir "$NSSDB" -force > /dev/null || :
        fi

        # register our PKCS11 module
        if [ -h "$CENTRIFY_PKCS11_MODULE_SYMLINK" ]; then
            rm $CENTRIFY_PKCS11_MODULE_SYMLINK
        fi

        ln -s $CENTRIFY_PKCS11_MODULE_PATH $CENTRIFY_PKCS11_MODULE_SYMLINK
        
        THIS_PK11INSTALL_BIN="$CIMS_INSTALL/bin/pk11install"
        THIS_MODULE_NAME="$CENTRIFY_PKCS11_MODULE"
        isThere=`modutil -rawlist -dbdir "$NSSDB" | grep "$THIS_MODULE_NAME" || echo NO`
        if [ "$isThere" == "NO" ]; then
            if [ -x "$THIS_PK11INSTALL_BIN" ]; then
                "$THIS_PK11INSTALL_BIN" -p "$NSSDB" "name=\"$THIS_MODULE_NAME\" library=$CENTRIFY_PKCS11_MODULE_SYMLINK" || :
            fi
        fi
        
        if [ -h "$SYSTEM_COOLKEY_SYMLINK" ]; then
            mv -f "$SYSTEM_COOLKEY_SYMLINK" "$SYSTEM_COOLKEY_SYMLINK_BACKUP"
            ln -s "$CENTRIFY_COOLKEY_PATH" "$SYSTEM_COOLKEY_SYMLINK"
        fi
        
        if [ -h "$SYSTEM_CKAPPLET_SYMLINK" ]; then
            mv -f "$SYSTEM_CKAPPLET_SYMLINK" "$SYSTEM_CKAPPLET_SYMLINK_BACKUP"
            ln -s "$CENTRIFY_CKAPPLET_PATH" "$SYSTEM_CKAPPLET_SYMLINK"
        fi
        
        # enable system smart card support  
        $CIMS_INSTALL/sbin/rhel_authconfig.sh smartcard enable
        
        ;;


    disable)
        check_root

        if $RPM --quiet -q coolkey; then
            # re-register original CoolKey module to NSS
            isThere=`modutil -rawlist -dbdir "$NSSDB" | grep "$COOLKEY_MODULE_NAME" || echo NO`
            if [ "$isThere" == "NO" ]; then
                THIS_PK11INSTALL_BIN="$CIMS_INSTALL/bin/pk11install"
                if [ -x "$THIS_PK11INSTALL_BIN" ]; then
                    "$THIS_PK11INSTALL_BIN" -p "$NSSDB" "name=\"$COOLKEY_MODULE_NAME\" library=libcoolkeypk11.so" > /dev/null 2>&1 || :
                fi
            fi
        fi

        # unregister our PKCS11 module
        # - taken from coolkey.spec
        CENTRIFY_PKCS11_MODULES_TO_REMOVE=`modutil -rawlist -dbdir "$NSSDB" | grep "$CENTRIFY_PKCS11_MODULE" | grep -o 'name=\".*\"' | sed "s/^name=//"`
        ORIG_IFS=$IFS
        IFS=$'\n'
        for c in $CENTRIFY_PKCS11_MODULES_TO_REMOVE; do
            c=`echo $c | xargs`
            modutil -delete $c -dbdir "$NSSDB" -force > /dev/null 2>&1 || :
        done
        IFS=$ORIG_IFS

        # restore backups
        if [ -f $PAM_PKCS11_BACKUP_FILE ] && [ ! -h $PAM_PKCS11_FILE ]; then
            mv $PAM_PKCS11_BACKUP_FILE $PAM_PKCS11_FILE > /dev/null        
        fi

        # handle smartcard-auth symlinks
        if [ -h /etc/pam.d/smartcard-auth ]; then
            if [ -f /etc/pam.d/smartcard-auth-ac.cdc ]; then
                rm /etc/pam.d/smartcard-auth-ac.cdc
            fi
            if [ -f /etc/pam.d/smartcard-auth-ac ]; then
                rm /etc/pam.d/smartcard-auth
                ln -s smartcard-auth-ac /etc/pam.d/smartcard-auth
            fi
        fi

        if [ -h "$SYSTEM_COOLKEY_SYMLINK_BACKUP" ]; then
            rm "$SYSTEM_COOLKEY_SYMLINK"
            mv -f "$SYSTEM_COOLKEY_SYMLINK_BACKUP" "$SYSTEM_COOLKEY_SYMLINK"
        fi
        
        if [ -h "$SYSTEM_CKAPPLET_SYMLINK_BACKUP" ]; then
            rm "$SYSTEM_CKAPPLET_SYMLINK"
            mv -f "$SYSTEM_CKAPPLET_SYMLINK_BACKUP" "$SYSTEM_CKAPPLET_SYMLINK"
        fi

        # disable system smart card support  
        $CIMS_INSTALL/sbin/rhel_authconfig.sh smartcard disable
        
        ;;
    *)
        echo "Unknown operation $OPERATION" > /dev/stderr
        usage
        exit -1
        ;;
esac

exit 0

